@@ -22,7 +22,7 @@
  * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  * SOFTWARE. */
 
-#include "options.h"
+#include "includes.h"
 #include "packet.h"
 #include "buffer.h"
 #include "session.h"
@@ -37,6 +37,8 @@
 #include "x11fwd.h"
 #include "agentfwd.h"
 
+/* Handles sessions (either shells or programs) requested by the client */
+
 static int sessioncommand(struct Channel *channel, struct ChanSess *chansess,
 		char iscmd);
 static int sessionpty(struct ChanSess * chansess);
@@ -53,6 +55,7 @@ struct SigMap {
 	char* name;
 };
 
+/* Mapping of signal values to ssh signal strings */
 struct SigMap signames[] = {
 	{SIGABRT, "ABRT"},
 	{SIGALRM, "ALRM"},
@@ -67,12 +70,14 @@ struct SigMap signames[] = {
 	{SIGTERM, "TERM"},
 	{SIGUSR1, "USR1"},
 	{SIGUSR2, "USR2"},
-	{0}
+	{0, NULL}
 };
 
 /* required to clear environment */
 extern char** environ;
 
+/* Set up the general chansession environment, in particular child-exit
+ * handling */
 void chansessinitialise() {
 
 	struct sigaction sa_chld;
@@ -90,11 +95,12 @@ void chansessinitialise() {
 	
 }
 
+/* handler for childs exiting, store the state for return to the client */
 static void sesssigchild_handler(int val) {
 
 	int status;
 	pid_t pid;
-	int i;
+	unsigned int i;
 	struct ChanSess * chansess;
 	struct sigaction sa_chld;
 	
@@ -128,6 +134,7 @@ static void sesssigchild_handler(int val) {
 	TRACE(("leave sigchld handler"));
 }
 
+/* send the exitstatus to the client */
 void send_msg_chansess_exitstatus(struct Channel * channel,
 		struct ChanSess * chansess) {
 
@@ -146,6 +153,7 @@ void send_msg_chansess_exitstatus(struct Channel * channel,
 
 }
 
+/* send the signal causing the exit to the client */
 void send_msg_chansess_exitsignal(struct Channel * channel,
 		struct ChanSess * chansess) {
 
@@ -184,7 +192,7 @@ void send_msg_chansess_exitsignal(struct Channel * channel,
 	encrypt_packet();
 }
 
-/* set up a session type channel */
+/* set up a session channel */
 void newchansess(struct Channel *channel) {
 
 	struct ChanSess *chansess;
@@ -223,11 +231,11 @@ void newchansess(struct Channel *channel) {
 
 }
 
-/* clean up a session type channel */
+/* clean a session channel */
 void closechansess(struct Channel *channel) {
 
 	struct ChanSess *chansess;
-	int i;
+	unsigned int i;
 	struct logininfo *li;
 
 	chansess = (struct ChanSess*)channel->typedata;
@@ -276,6 +284,8 @@ void closechansess(struct Channel *channel) {
 	TRACE(("leave closechansess"));
 }
 
+/* Handle requests for a channel. These can be execution requests,
+ * or x11/authagent forwarding. These are passed to appropriate handlers */
 void chansessionrequest(struct Channel *channel) {
 
 	unsigned char * type;
@@ -337,7 +347,7 @@ void chansessionrequest(struct Channel *channel) {
 }
 
 
-/* returns 0 on success, 1 otherwise */
+/* Send a signal to a session's process as requested by the client*/
 static int sessionsignal(struct ChanSess *chansess) {
 
 	int sig = 0;
@@ -346,7 +356,7 @@ static int sessionsignal(struct ChanSess *chansess) {
 
 	if (chansess->pid == 0) {
 		/* haven't got a process pid yet */
-		return 1;
+		return DROPBEAR_FAILURE;
 	}
 
 	signame = buf_getstring(ses.payload, NULL);
@@ -364,22 +374,23 @@ static int sessionsignal(struct ChanSess *chansess) {
 
 	if (sig == 0) {
 		/* failed */
-		return 1;
+		return DROPBEAR_FAILURE;
 	}
 			
 	if (kill(chansess->pid, sig) < 0) {
-		return 1;
+		return DROPBEAR_FAILURE;
 	} 
 
-	return 0;
+	return DROPBEAR_SUCCESS;
 }
 
-/* returns 0 on success, 1 on failure */
+/* Let the process know that the window size has changed, as notified from the
+ * client. Returns DROPBEAR_SUCCESS or DROPBEAR_FAILURE */
 static int sessionwinchange(struct ChanSess *chansess) {
 
 	if (chansess->master < 0) {
 		/* haven't got a pty yet */
-		return 1;
+		return DROPBEAR_FAILURE;
 	}
 			
 	chansess->termc = buf_getint(ses.payload);
@@ -390,12 +401,12 @@ static int sessionwinchange(struct ChanSess *chansess) {
 	pty_change_window_size(chansess->master, chansess->termr, chansess->termc,
 		chansess->termw, chansess->termh);
 
-	return 0;
+	return DROPBEAR_FAILURE;
 }
 
-
-
-/* returns 0 on success, 1 on failure */
+/* Set up a session pty which will be used to execute the shell or program.
+ * The pty is allocated now, and kept for when the shell/program executes.
+ * Returns DROPBEAR_SUCCESS or DROPBEAR_FAILURE */
 static int sessionpty(struct ChanSess * chansess) {
 
 	unsigned int termlen;
@@ -407,7 +418,7 @@ static int sessionpty(struct ChanSess * chansess) {
 	if (termlen > MAX_TERM_LEN) {
 		/* TODO send disconnect ? */
 		TRACE(("leave sessionpty: term len too long"));
-		return 1;
+		return DROPBEAR_FAILURE;
 	}
 	chansess->termc = buf_getint(ses.payload);
 	chansess->termr = buf_getint(ses.payload);
@@ -418,7 +429,7 @@ static int sessionpty(struct ChanSess * chansess) {
 	assert(chansess->master == -1); /* haven't already got one */
 	if (pty_allocate(&chansess->master, &chansess->slave, namebuf, 64) == 0) {
 		TRACE(("leave sessionpty: failed to allocate pty"));
-		return 1;
+		return DROPBEAR_FAILURE;
 	}
 	
 	chansess->tty = (char*)strdup(namebuf);
@@ -497,11 +508,13 @@ static int sessionpty(struct ChanSess * chansess) {
 	}
 
 	TRACE(("leave sessionpty"));
-	return 0;
-	
+	return DROPBEAR_SUCCESS;
 }
 
-/* returns 0 on sucesss, 1 otherwise */
+/* Handle a command request from the client. This is used for both shell
+ * and command-execution requests, and passes the command to
+ * noptycommand or ptycommand as appropriate.
+ * Returns DROPBEAR_SUCCESS or DROPBEAR_FAILURE */
 static int sessioncommand(struct Channel *channel, struct ChanSess *chansess,
 		char iscmd) {
 
@@ -511,7 +524,7 @@ static int sessioncommand(struct Channel *channel, struct ChanSess *chansess,
 
 	if (chansess->cmd != NULL) {
 		/* TODO - send error - multiple commands? */
-		return 1;
+		return DROPBEAR_FAILURE;
 	}
 
 	if (iscmd) {
@@ -520,7 +533,7 @@ static int sessioncommand(struct Channel *channel, struct ChanSess *chansess,
 
 		if (cmdlen > MAX_CMD_LEN) {
 			/* TODO - send error - too long ? */
-			return 1;
+			return DROPBEAR_FAILURE;
 		}
 	}
 
@@ -533,7 +546,9 @@ static int sessioncommand(struct Channel *channel, struct ChanSess *chansess,
 	}
 }
 
-/* returns 0 on success, 1 on fail */
+/* Execute a command and set up redirection of stdin/stdout/stderr without a
+ * pty.
+ * Returns DROPBEAR_SUCCESS or DROPBEAR_FAILURE */
 static int noptycommand(struct Channel *channel, struct ChanSess *chansess) {
 
 	int infds[2];
@@ -545,15 +560,15 @@ static int noptycommand(struct Channel *channel, struct ChanSess *chansess) {
 
 	/* redirect stdin/stdout/stderr */
 	if (pipe(infds) != 0)
-		return 1;
+		return DROPBEAR_FAILURE;
 	if (pipe(outfds) != 0)
-		return 1;
+		return DROPBEAR_FAILURE;
 	if (pipe(errfds) != 0)
-		return 1;
+		return DROPBEAR_FAILURE;
 
 	pid = fork();
 	if (pid < 0)
-		return 1;
+		return DROPBEAR_FAILURE;
 
 	if (!pid) {
 		/* child */
@@ -564,8 +579,8 @@ static int noptycommand(struct Channel *channel, struct ChanSess *chansess) {
 		if ((dup2(infds[FDIN], STDIN_FILENO) < 0) ||
 			(dup2(outfds[FDOUT], STDOUT_FILENO) < 0) ||
 			(dup2(errfds[FDOUT], STDERR_FILENO) < 0)) {
-			TRACE(("leave sessioncommand: error redirecting FDs"));
-			return 1;
+			TRACE(("leave noptycommand: error redirecting FDs"));
+			return DROPBEAR_FAILURE;
 		}
 
 		close(infds[FDOUT]);
@@ -580,7 +595,7 @@ static int noptycommand(struct Channel *channel, struct ChanSess *chansess) {
 
 	} else {
 		/* parent */
-		TRACE(("continue sessioncommand: parent"));
+		TRACE(("continue noptycommand: parent"));
 		chansess->pid = pid;
 
 		/* add a child pid */
@@ -606,10 +621,12 @@ static int noptycommand(struct Channel *channel, struct ChanSess *chansess) {
 #undef FDOUT
 
 	TRACE(("leave noptycommand"));
-	return 0;
+	return DROPBEAR_SUCCESS;
 }
 
-/* returns 0 on success, 1 on fail */
+/* Execute a command or shell within a pty environment, and set up
+ * redirection as appropriate.
+ * Returns DROPBEAR_SUCCESS or DROPBEAR_FAILURE */
 static int ptycommand(struct Channel *channel, struct ChanSess *chansess) {
 
 	pid_t pid;
@@ -621,7 +638,7 @@ static int ptycommand(struct Channel *channel, struct ChanSess *chansess) {
 	assert(chansess->master != -1 && chansess->tty != NULL);
 	pid = fork();
 	if (pid < 0)
-		return 1;
+		return DROPBEAR_FAILURE;
 
 	if (pid == 0) {
 		/* child */
@@ -634,8 +651,8 @@ static int ptycommand(struct Channel *channel, struct ChanSess *chansess) {
 		if ((dup2(chansess->slave, STDIN_FILENO) < 0) ||
 			(dup2(chansess->slave, STDERR_FILENO) < 0) ||
 			(dup2(chansess->slave, STDOUT_FILENO) < 0)) {
-			TRACE(("leave sessioncommand: error redirecting filedesc"));
-			return 1;
+			TRACE(("leave ptycommand: error redirecting filedesc"));
+			return DROPBEAR_FAILURE;
 		}
 
 		close(chansess->slave);
@@ -654,7 +671,7 @@ static int ptycommand(struct Channel *channel, struct ChanSess *chansess) {
 
 	} else {
 		/* parent */
-		TRACE(("continue sessioncommand: parent"));
+		TRACE(("continue ptycommand: parent"));
 		chansess->pid = pid;
 
 		/* add a child pid */
@@ -673,12 +690,13 @@ static int ptycommand(struct Channel *channel, struct ChanSess *chansess) {
 	}
 
 	TRACE(("leave ptycommand"));
-	return 0;
+	return DROPBEAR_SUCCESS;
 }
 
+/* Add the pid of a child to the list for exit-handling */
 static void addchildpid(struct ChanSess *chansess, pid_t pid) {
 
-	int i;
+	unsigned int i;
 	for (i = 0; i < ses.childpidsize; i++) {
 		if (ses.childpids[i].pid == -1) {
 			break;
@@ -703,7 +721,7 @@ static void execchild(struct ChanSess *chansess) {
 	char *argv[4];
 	char * usershell;
 	char * baseshell;
-	int i;
+	unsigned int i;
 
 	/* wipe the hostkey */
 	sign_key_free(ses.opts->hostkey);
@@ -713,8 +731,8 @@ static void execchild(struct ChanSess *chansess) {
 
 	/* close file descriptors except stdin/stdout/stderr
 	 * Need to be sure FDs are closed here to avoid reading files as root */
-	for (i = 3; i < ses.maxfd; i++) {
-		if (m_close(i) == -1) {
+	for (i = 3; i < (unsigned int)ses.maxfd; i++) {
+		if (m_close(i) == DROPBEAR_FAILURE) {
 			dropbear_exit("Error closing file desc");
 		}
 	}
