@@ -22,15 +22,9 @@
  * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  * SOFTWARE. */
 
-#include "libtomcrypt/mycrypt_custom.h"
-
 #ifndef _OPTIONS_H_
 #define _OPTIONS_H_
 
-#include "config.h"
-#include "debug.h"
-#include "includes.h"
-
 /******************************************************************
  * Define compile-time options below.
  ******************************************************************/
@@ -38,14 +32,16 @@
 #define DROPBEAR_PORT 22
 
 /* Default hostkey paths - these can be specified on the command line */
-#define DSS_PRIV_FILENAME "dropbear_dss_host_key"
-#define RSA_PRIV_FILENAME "dropbear_rsa_host_key"
+#define DSS_PRIV_FILENAME "/etc/dropbear_dss_host_key"
+#define RSA_PRIV_FILENAME "/etc/dropbear_rsa_host_key"
 
 /* Disable X11 Forwarding */
 /*#define DISABLE_X11FWD*/
 
 /* Disable TCP Fowarding */
-/*#define DISABLE_TCPFWD*/
+/* NOTE: TCP forwarding is still work-in-progress, -L forwarding should work,
+ * enable TCP forwarding at your own risk. */
+#define DISABLE_TCPFWD
 
 /* Disable Authentication Agent Forwarding */
 /*#define DISABLE_AGENTFWD*/
@@ -64,7 +60,7 @@
  * Disabling it here will just stop it from being used as the integrity portion
  * of the ssh protocol */
 #define DROPBEAR_SHA1_HMAC
-/*#define DROPBEAR_MD5_HMAC*/
+#define DROPBEAR_MD5_HMAC
 
 /* Hostkey/public key algorithms - at least one required, these are used
  * for hostkey as well as for verifying signatures with pubkey auth.
@@ -118,7 +114,7 @@
  * You shouldn't edit below here unless you know you need to.
  *******************************************************************/
 
-#define DROPBEAR_VERSION "0.32"
+#define DROPBEAR_VERSION "0.34"
 #define LOCAL_IDENT "SSH-2.0-dropbear_" DROPBEAR_VERSION
 #define PROGNAME "dropbear"
 
@@ -157,6 +153,9 @@
 /* SHA512 is 64 bytes == 512 bits */
 #define SHA512_HASH_SIZE 64
 
+/* largest of MD5 and SHA1 */
+#define MAX_MAC_LEN SHA1_HASH_SIZE
+
 
 #define MAX_KEY_LEN 24 /* 3DES requires a 24 byte key */
 #define MAX_IV_LEN 20 /* must be same as max blocksize, 
@@ -193,6 +192,16 @@
 #define MAX_CMD_LEN 1024 /* max length of a command */
 #define MAX_TERM_LEN 200 /* max length of TERM name */
 
+#define MAX_HOST_LEN 254 /* max hostname len for tcp fwding */
+
+#define DROPBEAR_MAX_PORTS 10 /* max number of ports which can be specified,
+								 ipv4 and ipv6 don't count twice */
+
 #define _PATH_TTY "/dev/tty"
 
+#ifdef DISABLE_TCPFWD
+#define DISABLE_LOCALTCPFWD
+#define DISABLE_REMOTETCPFWD
+#endif
+
 #endif /* _OPTIONS_H_ */
